from pprint import pprint
from argparse import ArgumentParser
import atexit
import os
import sys
import datetime
import time

from .Session import Session
from .SideCarFileReader import SideCarFileReader

from .Errors import TestSystemError
from .ResultCodes import ResultCodes

class TestWizard(object):
    def __enter__(self):
        parser = ArgumentParser()
        parser.add_argument("-s", "--sidecar", dest="sidecar", help="location of the sidecar file" )
        parser.add_argument("-t", "--testrun", dest="testrun", help="testrun" )
        args = parser.parse_args()

        scriptFilePath = self.__getScriptFilePath()
        sideCarFilePath = self.__getSideCarFilePath(args, scriptFilePath)
        workingDirectory = self.__getWorkingDirectory()

        metadata = SideCarFileReader.read(sideCarFilePath)
        if not 'outputFolder' in metadata:
            metadata["outputFolder"] = self.__constructOutputFolder(scriptFilePath)

        self.session = Session(scriptFilePath, workingDirectory, metadata, args.testrun)

        return self

    def __exit__(self, type, value, traceback):
        if isinstance(value, TestSystemError):
            self.session.setResult(ResultCodes.SYSTEMERROR, str(value))
        elif isinstance(value, Exception):
            self.session.setResult(ResultCodes.SCRIPTERROR, str(value))

        self.session.dispose()

    def __getScriptFilePath(self):
        return os.path.abspath(sys.argv[0])

    def __getWorkingDirectory(self):
        return os.getcwd()

    def __getSideCarFilePath(self, args, scriptFilePath):
        if args.sidecar is not None:
            return args.sidecar
        
        return os.path.splitext(scriptFilePath)[0] + ".json"

    def __constructOutputFolder(self, scriptFilePath):
        strDate = datetime.datetime.now().isoformat()
        timestr = time.strftime("%Y%m%d-%H%M%S")
        strDate.replace("-", "")
        strDate.replace(":", "")
        strDate.replace(".", "")

        return os.getcwd() + "/Runs/" + os.path.splitext(os.path.basename(scriptFilePath))[0] + "_" + timestr



import time
from enum import Enum
from collections import namedtuple

from .RobotClient import RobotClient
from .ResultCodes import ResultCodes
from .models.TestResult import TestResult

class Session():
    def __init__(self, scriptFilePath, workingDirectory, metadata, testRunId):
        self.metadata = metadata

        self.parameters = {}
        for parameter in metadata.get("parameters", {}):
            self.parameters[parameter['name']] = parameter['value']

        self.customProperties = metadata.get("customProperties", {})

        self.robot = RobotClient("localhost:8500")

        if testRunId is None:
            self.testRunId = self.robot.createTestRun(scriptFilePath, workingDirectory, metadata)
            self.__isSelfManaged = True
        else:
            self.testRunId = testRunId
            self.__isSelfManaged = False

        self.info = self.robot.getSessionInfo(self.testRunId)

        self.__failCount = 0
        self.__errorCount = 0
        self.__results = []

        self.__isDisposed = False

    def sleep(self, seconds):
        time.sleep(seconds)

    @property
    def failCount(self):
        self.__throwIfDisposed()

        return self.__failCount > 0

    @property
    def errorCount(self):
        self.__throwIfDisposed()

        return self.__errorCount > 0

    def addPass(self, message):
        self.__throwIfDisposed()

        self.__results.append(TestResult(ResultCodes.PASS, message))

    def addFail(self, message):
        self.__throwIfDisposed()

        self.__failCount += 1
        self.__results.append(TestResult(ResultCodes.FAIL, message))

    def addError(self, message):
        self.__throwIfDisposed()

        self.__errorCount += 1
        self.__results.append(TestResult(ResultCodes.SCRIPTERROR, message))

    def setResult(self, result, message):
        self.__throwIfDisposed()

        if not isinstance(result, ResultCodes):
            raise Exception(f"Result '{str(result)}' is not of type ResultCodes.")
        
        resultValue = result.value
        if resultValue == ResultCodes.FAIL.value:
            self.__failCount += 1
        elif resultValue == ResultCodes.SCRIPTERROR.value:
            self.__errorCount += 1
        elif resultValue == ResultCodes.SYSTEMERROR.value:
            self.__errorCount += 1

        self.robot.postTestResult(self.testRunId, [ TestResult(result, message) ])

    @property
    def hasFails(self):
        self.__throwIfDisposed()

        return self.__failCount > 0

    @property
    def hasErrors(self):
        self.__throwIfDisposed()

        return self.__errorCount > 0

    def dispose(self):
        self.__isDisposed = True

        if len(self.__results) > 0:
            self.robot.postTestResult(self.testRunId, self.__results)

        if self.__isSelfManaged is True:
            self.robot.tearDown(self.testRunId)

        self.__failCount = None
        self.__errorCount = None
        self.__results = None

        self.testRunId = None
        self.__isSelfManaged = True

        self.robot = None

    def __throwIfDisposed(self):
        if self.__isDisposed is True:
            print("Cannot access a disposed object")
            raise Exception("Cannot access a disposed object.")


from setuptools import setup, find_namespace_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="testwizard.core",
    version="3.2.9",
    author="Eurofins Digital Testing - Belgium",
    author_email="support-be@eurofins.com",
    description="Testwizard core",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_namespace_packages(),
    install_requires=[
          'jsons',
    ],
    classifiers=[
        "Programming Language :: Python :: 3.3",
        "Development Status :: 5 - Production/Stable",
        "Operating System :: OS Independent",
    ],
)





